@@ -4,8 +4,10 @@
 
 #include <string.h>
 #include <stdlib.h>
+
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <libgnome/gnome-i18n.h>
+#include <gdk/gdkx.h>
 
 #include "clock.h"
 #include "clock-face.h"
@@ -19,7 +21,6 @@ G_DEFINE_TYPE (ClockLocationTile, clock_location_tile, GTK_TYPE_ALIGNMENT)
 
 enum {
 	TILE_PRESSED,
-	TIMEZONE_SET,
 	NEED_CLOCK_FORMAT,
 	LAST_SIGNAL
 };
@@ -106,14 +107,6 @@ clock_location_tile_class_init (ClockLocationTileClass *this_class)
 					      NULL,
 					      g_cclosure_marshal_VOID__VOID,
 					      G_TYPE_NONE, 0);
-	signals[TIMEZONE_SET] = g_signal_new ("timezone-set",
-					      G_TYPE_FROM_CLASS (g_obj_class),
-					      G_SIGNAL_RUN_FIRST,
-					      G_STRUCT_OFFSET (ClockLocationTileClass, timezone_set),
-					      NULL,
-					      NULL,
-					      g_cclosure_marshal_VOID__VOID,
-					      G_TYPE_NONE, 0);
 	signals[NEED_CLOCK_FORMAT] = g_signal_new ("need-clock-format",
 						   G_TYPE_FROM_CLASS (g_obj_class),
 						   G_SIGNAL_RUN_LAST,
@@ -180,13 +173,9 @@ press_on_tile      (GtkWidget             *widget,
 static void
 make_current_cb (gpointer data, GError *error)
 {
-	ClockLocationTile *tile = data;
 	GtkWidget *dialog;
 
-        if (error == NULL) {
-		g_signal_emit (tile, signals[TIMEZONE_SET], 0);
-        }
-        else {
+        if (error) {
                 dialog = gtk_message_dialog_new (NULL,
                                                  0,
                                                  GTK_MESSAGE_ERROR,
@@ -205,8 +194,17 @@ static void
 make_current (GtkWidget *widget, ClockLocationTile *tile)
 {
         ClockLocationTilePrivate *priv = PRIVATE (tile);
+	GtkWidget *toplevel;
+	guint xid;
 
-	clock_location_make_current (priv->location, 
+	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (tile));
+	if (toplevel && toplevel->window)
+		xid = GDK_WINDOW_XWINDOW (toplevel->window);
+	else
+		xid = 0;
+
+	clock_location_make_current (priv->location,
+				     xid,
 				     (GFunc)make_current_cb, tile, NULL);
 }
 
@@ -232,7 +230,10 @@ enter_or_leave_tile (GtkWidget             *widget,
 	if (event->type == GDK_ENTER_NOTIFY) {
 		gint can_set;
 
-		can_set = can_set_system_timezone ();
+		if (clock_location_is_current_timezone (priv->location))
+			can_set = 2;
+		else
+			can_set = can_set_system_timezone ();
 		if (can_set != 0) {
 			gtk_label_set_markup (GTK_LABEL (priv->current_label),
 						can_set == 1 ?
@@ -269,7 +270,6 @@ clock_location_tile_fill (ClockLocationTile *this)
         GtkWidget *alignment;
         GtkWidget *tile;
         GtkWidget *head_section;
-	gint can_set;
 
         priv->box = gtk_event_box_new ();
 
@@ -307,19 +307,13 @@ clock_location_tile_fill (ClockLocationTile *this)
         gtk_box_pack_start (GTK_BOX (box), align, FALSE, FALSE, 0);
         gtk_box_pack_start (GTK_BOX (box), priv->time_label, FALSE, FALSE, 0);
 
-	can_set = can_set_system_timezone ();
-
         priv->current_button = gtk_button_new ();
+	/* The correct label is set on EnterNotify events */
 	priv->current_label = gtk_label_new ("");
-	if (can_set != 0) {
-		gtk_label_set_markup (GTK_LABEL (priv->current_label),
-				can_set == 1 ?  
-					_("<small>Set...</small>") :
-					_("<small>Set</small>"));
-	}
         gtk_widget_show (priv->current_label);
         gtk_widget_set_no_show_all (priv->current_button, TRUE);
         gtk_container_add (GTK_CONTAINER (priv->current_button), priv->current_label);
+	/* FIXME: this needs to talk about locations, not timezones */
         gtk_widget_set_tooltip_text (priv->current_button, _("Set as current timezone for this computer"));
 
 	priv->current_marker = gtk_image_new_from_icon_name ("go-home", GTK_ICON_SIZE_BUTTON);
@@ -572,7 +566,8 @@ weather_info_setup_tooltip (WeatherInfo *info, ClockLocation *location, GtkToolt
 {
         GdkPixbuf *pixbuf = NULL;
         GtkIconTheme *theme = NULL;
-	const gchar *conditions, *temp, *apparent, *wind;
+	const gchar *conditions, *wind;
+	gchar *temp, *apparent;
 	gchar *line1, *line2, *line3, *line4, *tip;
 	const gchar *icon_name;
 	const gchar *sys_timezone;
@@ -591,14 +586,18 @@ weather_info_setup_tooltip (WeatherInfo *info, ClockLocation *location, GtkToolt
 	else
 		line1 = g_strdup (weather_info_get_sky (info));
 
-	temp = weather_info_get_temp_summary (info);
-	apparent = weather_info_get_apparent (info);
+	/* we need to g_strdup() since both functions return the same address
+	 * of a static buffer */
+	temp = g_strdup (weather_info_get_temp (info));
+	apparent = g_strdup (weather_info_get_apparent (info));
 	if (strcmp (apparent, temp) != 0 &&
 	    /* FMQ: it's broken to read from another module's translations; add some API to libgweather. */
             strcmp (apparent, dgettext ("gnome-applets-2.0", "Unknown")) != 0)
 		line2 = g_strdup_printf (_("%s, feels like %s"), temp, apparent);
 	else
 		line2 = g_strdup (temp);
+	g_free (temp);
+	g_free (apparent);
 
 	wind = weather_info_get_wind (info);
         if (strcmp (apparent, dgettext ("gnome-applets-2.0", "Unknown")) != 0)
