@@ -26,13 +26,17 @@
 
 #include <string.h>
 
-#include <glib/gi18n-lib.h>
+#include <glib/gi18n.h>
+#include <gio/gio.h>
 #include <gtk/gtk.h>
 
 #include <libgnomeui/gnome-icon-entry.h>
 #include <libgnomeui/gnome-help.h>
-#include <libgnomevfs/gnome-vfs.h>
+
+#include <libpanel-util/panel-keyfile.h>
+
 #include "panel-ditem-editor.h"
+#include "panel-icon-names.h"
 #include "panel-util.h"
 #include "panel-marshal.h"
 
@@ -50,6 +54,7 @@ struct _PanelDItemEditorPrivate
 	guint     save_timeout;
 
 	char     *uri; /* file location */
+	gboolean  type_directory;
 	gboolean  new_file;
 	gboolean  combo_setuped;
 
@@ -80,8 +85,8 @@ struct _PanelDItemEditorPrivate
 	char *icon_theme_dir;
 };
 
-/* Time in milliseconds after which we save the file on the disk */
-#define SAVE_FREQUENCY 2000
+/* Time in seconds after which we save the file on the disk */
+#define SAVE_FREQUENCY 2
 
 enum {
 	REVERT_BUTTON
@@ -112,7 +117,7 @@ static ComboItem type_items [] = {
 	  PANEL_DITEM_EDITOR_TYPE_APPLICATION          },
 	{ N_("Application in Terminal"), "Application",
 	  PANEL_DITEM_EDITOR_TYPE_TERMINAL_APPLICATION },
-	{ N_("File"),                    "Link",
+	{ N_("Location"),                "Link",
 	  PANEL_DITEM_EDITOR_TYPE_LINK                 },
 	/* FIXME: hack hack hack: we will remove this item from the combo
 	 * box if we show it */
@@ -157,6 +162,7 @@ enum {
 	PROP_0,
 	PROP_KEYFILE,
 	PROP_URI,
+	PROP_TYPEDIRECTORY
 };
 
 static guint ditem_edit_signals[LAST_SIGNAL] = { 0 };
@@ -171,6 +177,10 @@ static void type_combo_changed (PanelDItemEditor *dialog);
 
 static void response_cb (GtkDialog *dialog,
 			 gint       response_id);
+
+static void setup_icon_entry (PanelDItemEditor *dialog,
+			      const char       *icon_name);
+
 static gboolean panel_ditem_editor_save         (PanelDItemEditor *dialog,
 						 gboolean          report_errors);
 static gboolean panel_ditem_editor_save_timeout (gpointer data);
@@ -183,6 +193,10 @@ static gboolean panel_ditem_editor_load_uri (PanelDItemEditor  *dialog,
 static void panel_ditem_editor_set_key_file (PanelDItemEditor *dialog,
 					     GKeyFile         *key_file);
 
+static gboolean panel_ditem_editor_get_type_directory (PanelDItemEditor *dialog);
+static void panel_ditem_editor_set_type_directory (PanelDItemEditor *dialog,
+						   gboolean          type_directory);
+
 
 static PanelDItemEditorType
 map_type_from_desktop_item (const char *type,
@@ -210,8 +224,9 @@ panel_ditem_editor_constructor (GType                  type,
 {
 	GObject          *obj;
 	PanelDItemEditor *dialog;
-	GnomeVFSURI      *vfs_uri;
+	GFile            *file;
 	gboolean          loaded;
+	char             *desktop_type;
 
 	obj = G_OBJECT_CLASS (panel_ditem_editor_parent_class)->constructor (type,
 									     n_construct_properties,
@@ -225,29 +240,41 @@ panel_ditem_editor_constructor (GType                  type,
 		dialog->priv->free_key_file = FALSE;
 		loaded = TRUE;
 	} else {
-		dialog->priv->key_file = panel_util_key_file_new_desktop ();
+		dialog->priv->key_file = panel_key_file_new_desktop ();
+		if (dialog->priv->type_directory)
+			panel_key_file_set_string (dialog->priv->key_file,
+						   "Type", "Directory");
 		dialog->priv->free_key_file = TRUE;
 		loaded = FALSE;
 	}
 
 	if (!loaded && dialog->priv->uri) {
-		vfs_uri = gnome_vfs_uri_new (dialog->priv->uri);
-		if (gnome_vfs_uri_exists (vfs_uri)) {
+		file = g_file_new_for_uri (dialog->priv->uri);
+		if (g_file_query_exists (file, NULL)) {
 			//FIXME what if there's an error?
 			panel_ditem_editor_load_uri (dialog, NULL);
 			dialog->priv->new_file = FALSE;
 		} else {
 			dialog->priv->new_file = TRUE;
 		}
-		gnome_vfs_uri_unref (vfs_uri);
+		g_object_unref (file);
 	} else {
 		dialog->priv->new_file = !loaded;
 	}
 
 	dialog->priv->dirty = FALSE;
 
+	desktop_type = panel_key_file_get_string (dialog->priv->key_file,
+						  "Type");
+	if (desktop_type && !strcmp (desktop_type, "Directory"))
+		dialog->priv->type_directory = TRUE;
+	g_free (desktop_type);
+
 	panel_ditem_editor_setup_ui (dialog);
 
+	if (dialog->priv->new_file)
+		setup_icon_entry (dialog, NULL);
+
 	return obj;
 }
 
@@ -270,6 +297,9 @@ panel_ditem_editor_get_property (GObject    *object,
 	case PROP_URI:
 		g_value_set_string (value, panel_ditem_editor_get_uri (dialog));
 		break;
+	case PROP_TYPEDIRECTORY:
+		g_value_set_boolean (value, panel_ditem_editor_get_type_directory (dialog));
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -290,10 +320,16 @@ panel_ditem_editor_set_property (GObject       *object,
 
 	switch (prop_id) {
 	case PROP_KEYFILE:
-		panel_ditem_editor_set_key_file (dialog, g_value_get_pointer (value));
+		panel_ditem_editor_set_key_file (dialog,
+						 g_value_get_pointer (value));
 		break;
 	case PROP_URI:
-		panel_ditem_editor_set_uri (dialog, g_value_get_string (value));
+		panel_ditem_editor_set_uri (dialog,
+					    g_value_get_string (value));
+		break;
+	case PROP_TYPEDIRECTORY:
+		panel_ditem_editor_set_type_directory (dialog,
+						       g_value_get_boolean (value));
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -450,6 +486,15 @@ panel_ditem_editor_class_init (PanelDItemEditorClass *class)
 				     "The URI of the .desktop file",
 				     NULL,
 				     G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+
+	g_object_class_install_property (
+		gobject_class,
+		PROP_TYPEDIRECTORY,
+		g_param_spec_boolean ("type-directory",
+				      "Type Directory",
+				      "Whether the edited file is a .directory file or not",
+				      FALSE,
+				      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 }
 
 static GtkWidget *
@@ -541,6 +586,9 @@ panel_ditem_editor_get_item_type (PanelDItemEditor *dialog)
 	GtkTreeModel         *model;
 	PanelDItemEditorType  type;
 
+	if (dialog->priv->type_directory)
+		return PANEL_DITEM_EDITOR_TYPE_DIRECTORY;
+
 	if (!gtk_combo_box_get_active_iter (GTK_COMBO_BOX (dialog->priv->type_combo),
 					    &iter))
 		return PANEL_DITEM_EDITOR_TYPE_NULL;
@@ -563,7 +611,7 @@ panel_ditem_editor_make_ui (PanelDItemEditor *dialog)
 	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
 
 	priv->table = gtk_table_new (4, 3, FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER (priv->table), 12);
+	gtk_container_set_border_width (GTK_CONTAINER (priv->table), 5);
 	gtk_table_set_row_spacings (GTK_TABLE (priv->table), 6);
 	gtk_table_set_col_spacings (GTK_TABLE (priv->table), 12);
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
@@ -655,6 +703,8 @@ panel_ditem_editor_setup_ui (PanelDItemEditor *dialog)
 		gtk_widget_hide (priv->close_button);
 		gtk_widget_show (priv->cancel_button);
 		gtk_widget_show (priv->ok_button);
+		gtk_dialog_set_default_response (GTK_DIALOG (dialog),
+						 GTK_RESPONSE_OK);
 
 		if (!priv->combo_setuped) {
 			setup_combo (priv->type_combo,
@@ -665,13 +715,15 @@ panel_ditem_editor_setup_ui (PanelDItemEditor *dialog)
 
 		gtk_combo_box_set_active (GTK_COMBO_BOX (priv->type_combo), 0);
 
-		show_combo = TRUE;
+		show_combo = !priv->type_directory;
 	} else {
 
 		gtk_widget_show (priv->revert_button);
 		gtk_widget_show (priv->close_button);
 		gtk_widget_hide (priv->cancel_button);
 		gtk_widget_hide (priv->ok_button);
+		gtk_dialog_set_default_response (GTK_DIALOG (dialog),
+						 GTK_RESPONSE_CLOSE);
 
 		show_combo = (type != PANEL_DITEM_EDITOR_TYPE_LINK) &&
 			     (type != PANEL_DITEM_EDITOR_TYPE_DIRECTORY);
@@ -715,8 +767,6 @@ panel_ditem_editor_setup_ui (PanelDItemEditor *dialog)
 				break;
 			}
 		} while (gtk_tree_model_iter_next (model, &iter));
-
-		gtk_widget_grab_focus (priv->type_combo);
 	} else if (type == PANEL_DITEM_EDITOR_TYPE_DIRECTORY) {
 		table_attach_label (GTK_TABLE (priv->table), priv->name_label,
 				    1, 2, 0, 1);
@@ -727,8 +777,6 @@ panel_ditem_editor_setup_ui (PanelDItemEditor *dialog)
 				    1, 2, 1, 2);
 		table_attach_entry (GTK_TABLE (priv->table), priv->comment_entry,
 				    2, 3, 1, 2);
-
-		gtk_widget_grab_focus (priv->name_entry);
 	} else {
 		table_attach_label (GTK_TABLE (priv->table), priv->name_label,
 				    1, 2, 0, 1);
@@ -744,8 +792,6 @@ panel_ditem_editor_setup_ui (PanelDItemEditor *dialog)
 				    1, 2, 2, 3);
 		table_attach_entry (GTK_TABLE (priv->table), priv->comment_entry,
 				    2, 3, 2, 3);
-
-		gtk_widget_grab_focus (priv->name_entry);
 	}
 
 	type_combo_changed (dialog);
@@ -762,6 +808,8 @@ panel_ditem_editor_setup_ui (PanelDItemEditor *dialog)
 	gtk_container_set_focus_chain (GTK_CONTAINER (priv->table),
 				       focus_chain);
 	g_list_free (focus_chain);
+
+	gtk_widget_grab_focus (priv->name_entry);
 }
 
 /*
@@ -776,9 +824,10 @@ panel_ditem_editor_changed (PanelDItemEditor *dialog)
 		if (dialog->priv->save_timeout != 0)
 			g_source_remove (dialog->priv->save_timeout);
 
-		dialog->priv->save_timeout = g_timeout_add (SAVE_FREQUENCY,
-							    panel_ditem_editor_save_timeout,
-							    dialog);
+		dialog->priv->save_timeout = g_timeout_add_seconds (
+							SAVE_FREQUENCY,
+							panel_ditem_editor_save_timeout,
+							dialog);
 
 		/* We can revert to the original state */
 		if (dialog->priv->revert_key_file != NULL)
@@ -791,6 +840,17 @@ panel_ditem_editor_changed (PanelDItemEditor *dialog)
 	g_signal_emit (G_OBJECT (dialog), ditem_edit_signals[CHANGED], 0);
 }
 
+static void
+panel_ditem_editor_activated (PanelDItemEditor *dialog)
+{
+	if (GTK_WIDGET_VISIBLE (dialog->priv->ok_button))
+		gtk_dialog_response (GTK_DIALOG (dialog),
+				     GTK_RESPONSE_OK);
+	else if (GTK_WIDGET_VISIBLE (dialog->priv->close_button))
+		gtk_dialog_response (GTK_DIALOG (dialog),
+				     GTK_RESPONSE_CLOSE);
+}
+
 static void
 panel_ditem_editor_name_changed (PanelDItemEditor *dialog)
 {
@@ -799,11 +859,11 @@ panel_ditem_editor_name_changed (PanelDItemEditor *dialog)
 	name = gtk_entry_get_text (GTK_ENTRY (dialog->priv->name_entry));
 
 	if (name && name[0])
-		panel_util_key_file_set_locale_string (dialog->priv->key_file,
-						       "Name", name);
+		panel_key_file_set_locale_string (dialog->priv->key_file,
+						  "Name", name);
 	else
-		panel_util_key_file_remove_locale_key (dialog->priv->key_file,
-						       "Name");
+		panel_key_file_remove_all_locale_key (dialog->priv->key_file,
+						      "Name");
 
 	g_signal_emit (G_OBJECT (dialog), ditem_edit_signals[NAME_CHANGED], 0,
 		       name);
@@ -814,6 +874,8 @@ panel_ditem_editor_command_changed (PanelDItemEditor *dialog)
 {
 	PanelDItemEditorType  type;
 	const char           *exec_or_uri;
+	GtkIconTheme         *icon_theme;
+	char                 *icon;
 
 	exec_or_uri = gtk_entry_get_text (GTK_ENTRY (dialog->priv->command_entry));
 
@@ -825,18 +887,34 @@ panel_ditem_editor_command_changed (PanelDItemEditor *dialog)
 	switch (type) {
 	case PANEL_DITEM_EDITOR_TYPE_APPLICATION:
 	case PANEL_DITEM_EDITOR_TYPE_TERMINAL_APPLICATION:
-		panel_util_key_file_remove_key (dialog->priv->key_file, "URL");
-		panel_util_key_file_set_string (dialog->priv->key_file, "Exec",
-						exec_or_uri);
+		panel_key_file_remove_key (dialog->priv->key_file, "URL");
+		panel_key_file_set_string (dialog->priv->key_file, "Exec",
+					   exec_or_uri);
+
+		icon_theme = gtk_icon_theme_get_for_screen (gtk_widget_get_screen (GTK_WIDGET (dialog)));
+		icon = guess_icon_from_exec (icon_theme,
+					     dialog->priv->key_file);
+		if (icon) {
+			char *current;
+
+			current = panel_key_file_get_locale_string (dialog->priv->key_file,
+								    "Icon");
+
+			if (!current || strcmp (icon, current))
+				setup_icon_entry (dialog, icon);
+
+			g_free (current);
+			g_free (icon);
+		}
 		break;
 	case PANEL_DITEM_EDITOR_TYPE_LINK:
-		panel_util_key_file_remove_key (dialog->priv->key_file, "Exec");
-		panel_util_key_file_set_string (dialog->priv->key_file, "URL",
-						exec_or_uri);
+		panel_key_file_remove_key (dialog->priv->key_file, "Exec");
+		panel_key_file_set_string (dialog->priv->key_file, "URL",
+					   exec_or_uri);
 		break;
 	default:
-		panel_util_key_file_remove_key (dialog->priv->key_file, "Exec");
-		panel_util_key_file_remove_key (dialog->priv->key_file, "URL");
+		panel_key_file_remove_key (dialog->priv->key_file, "Exec");
+		panel_key_file_remove_key (dialog->priv->key_file, "URL");
 	}
 
 	g_signal_emit (G_OBJECT (dialog), ditem_edit_signals[COMMAND_CHANGED],
@@ -851,11 +929,11 @@ panel_ditem_editor_comment_changed (PanelDItemEditor *dialog)
 	comment = gtk_entry_get_text (GTK_ENTRY (dialog->priv->comment_entry));
 
 	if (comment && comment[0])
-		panel_util_key_file_set_locale_string (dialog->priv->key_file,
-						       "Comment", comment);
+		panel_key_file_set_locale_string (dialog->priv->key_file,
+						  "Comment", comment);
 	else
-		panel_util_key_file_remove_locale_key (dialog->priv->key_file,
-						       "Comment");
+		panel_key_file_remove_all_locale_key (dialog->priv->key_file,
+						      "Comment");
 
 	g_signal_emit (G_OBJECT (dialog), ditem_edit_signals[COMMENT_CHANGED],
 		       0, comment);
@@ -876,19 +954,22 @@ panel_ditem_editor_icon_changed (PanelDItemEditor *dialog)
 		 * another icon from the theme.  See bug #119208 */
 		char *dn = g_path_get_dirname (file);
 		if (dialog->priv->icon_theme_dir != NULL &&
-		    strcmp (dn, dialog->priv->icon_theme_dir) == 0)
-			icon = g_path_get_basename (file);
-		else
+		    strcmp (dn, dialog->priv->icon_theme_dir) == 0) {
+			char *buffer;
+			buffer = g_path_get_basename (file);
+			icon = panel_util_icon_remove_extension (buffer);
+			g_free (buffer);
+		} else
 			icon = g_strdup (file);
 		g_free (dn);
 	}
 
 	if (icon)
-		panel_util_key_file_set_locale_string (dialog->priv->key_file,
-						       "Icon", icon);
+		panel_key_file_set_locale_string (dialog->priv->key_file,
+						  "Icon", icon);
 	else
-		panel_util_key_file_remove_locale_key (dialog->priv->key_file,
-						       "Icon");
+		panel_key_file_remove_all_locale_key (dialog->priv->key_file,
+						      "Icon");
 
 	g_signal_emit (G_OBJECT (dialog), ditem_edit_signals[ICON_CHANGED], 0,
 		       file);
@@ -901,12 +982,15 @@ command_browse_chooser_response (GtkFileChooser   *chooser,
 				 PanelDItemEditor *dialog)
 {
 	char *uri;
+	char *text;
 
 	if (response_id == GTK_RESPONSE_ACCEPT) {
 		switch (panel_ditem_editor_get_item_type (dialog)) {
 		case PANEL_DITEM_EDITOR_TYPE_APPLICATION:
 		case PANEL_DITEM_EDITOR_TYPE_TERMINAL_APPLICATION:
-			uri = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser));
+			text = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser));
+			uri = panel_util_make_exec_uri_for_desktop (text);
+			g_free (text);
 			break;
 		case PANEL_DITEM_EDITOR_TYPE_LINK:
 			uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (chooser));
@@ -995,10 +1079,10 @@ panel_ditem_editor_connect_signals (PanelDItemEditor *dialog)
 
 #define CONNECT_CHANGED(widget, callback) \
 	g_signal_connect_swapped (G_OBJECT (widget), "changed", \
-				  G_CALLBACK (panel_ditem_editor_changed), \
+				  G_CALLBACK (callback), \
 				  dialog); \
 	g_signal_connect_swapped (G_OBJECT (widget), "changed", \
-				  G_CALLBACK (callback), \
+				  G_CALLBACK (panel_ditem_editor_changed), \
 				  dialog);
 
 	CONNECT_CHANGED (priv->type_combo, type_combo_changed);
@@ -1007,6 +1091,16 @@ panel_ditem_editor_connect_signals (PanelDItemEditor *dialog)
 	CONNECT_CHANGED (priv->comment_entry, panel_ditem_editor_comment_changed);
 	CONNECT_CHANGED (priv->icon_entry, panel_ditem_editor_icon_changed);
 
+	g_signal_connect_swapped (priv->name_entry, "activate",
+				  G_CALLBACK (panel_ditem_editor_activated),
+				  dialog);
+	g_signal_connect_swapped (priv->command_entry, "activate",
+				  G_CALLBACK (panel_ditem_editor_activated),
+				  dialog);
+	g_signal_connect_swapped (priv->comment_entry, "activate",
+				  G_CALLBACK (panel_ditem_editor_activated),
+				  dialog);
+
 	g_signal_connect_swapped (priv->command_browse_button, "clicked",
 				  G_CALLBACK (command_browse_button_clicked),
 				  dialog);
@@ -1036,6 +1130,7 @@ panel_ditem_editor_init (PanelDItemEditor *dialog)
 	priv->dirty = FALSE;
 	priv->save_timeout = 0;
 	priv->uri = NULL;
+	priv->type_directory = FALSE;
 	priv->new_file = TRUE;
 	priv->save_uri = NULL;
 	priv->save_uri_data = NULL;
@@ -1055,29 +1150,37 @@ type_combo_changed (PanelDItemEditor *dialog)
 
 	switch (panel_ditem_editor_get_item_type (dialog)) {
 	case PANEL_DITEM_EDITOR_TYPE_APPLICATION:
-		text = _("_Command:");
-		panel_util_key_file_set_string (dialog->priv->key_file,
-						"Type", "Application");
-		panel_util_key_file_set_boolean (dialog->priv->key_file,
-						 "Terminal", FALSE);
+		text = _("Comm_and:");
+		if (dialog->priv->combo_setuped) {
+			panel_key_file_set_string (dialog->priv->key_file,
+						   "Type", "Application");
+			panel_key_file_set_boolean (dialog->priv->key_file,
+						    "Terminal", FALSE);
+		}
 		break;
 	case PANEL_DITEM_EDITOR_TYPE_TERMINAL_APPLICATION:
 		text = _("_Command:");
-		panel_util_key_file_set_string (dialog->priv->key_file,
-						"Type", "Application");
-		panel_util_key_file_set_boolean (dialog->priv->key_file,
-						 "Terminal", TRUE);
+		if (dialog->priv->combo_setuped) {
+			panel_key_file_set_string (dialog->priv->key_file,
+						   "Type", "Application");
+			panel_key_file_set_boolean (dialog->priv->key_file,
+						    "Terminal", TRUE);
+		}
 		break;
 	case PANEL_DITEM_EDITOR_TYPE_LINK:
 		text = _("_Location:");
-		panel_util_key_file_set_string (dialog->priv->key_file,
-						"Type", "Link");
-		panel_util_key_file_remove_key (dialog->priv->key_file,
-						"Terminal");
+		if (dialog->priv->combo_setuped) {
+			panel_key_file_set_string (dialog->priv->key_file,
+						   "Type", "Link");
+			panel_key_file_remove_key (dialog->priv->key_file,
+						   "Terminal");
+		}
 		break;
 	case PANEL_DITEM_EDITOR_TYPE_DIRECTORY:
-		panel_util_key_file_set_string (dialog->priv->key_file,
-						"Type", "Directory");
+		if (dialog->priv->combo_setuped) {
+			panel_key_file_set_string (dialog->priv->key_file,
+						   "Type", "Directory");
+		}
 		return;
 	default:
 		g_assert_not_reached ();
@@ -1094,6 +1197,47 @@ type_combo_changed (PanelDItemEditor *dialog)
 	update_chooser_for_type (dialog);
 }
 
+static void
+setup_icon_entry (PanelDItemEditor *dialog,
+		  const char       *icon_name)
+{
+	char *buffer;
+	char *tmpstr;
+
+	if (!icon_name || icon_name[0] == '\0') {
+		if (dialog->priv->type_directory) {
+			buffer = g_strdup (PANEL_ICON_FOLDER);
+		} else {
+			buffer = g_strdup (PANEL_ICON_LAUNCHER);
+		}
+	} else {
+		buffer = g_strdup (icon_name);
+	}
+
+	tmpstr = panel_find_icon (gtk_icon_theme_get_for_screen (gtk_widget_get_screen (GTK_WIDGET (dialog))),
+				  buffer, 48);
+
+	g_free (dialog->priv->icon_theme_dir);
+	if (tmpstr != NULL && buffer != NULL && !g_path_is_absolute (buffer)) {
+		/* this is a themed icon, see bug #119208 */
+		dialog->priv->icon_theme_dir = g_path_get_dirname (tmpstr);
+		/* FIXME: what about theme changes when the dialog is up */
+	} else {
+		/* use the default pixmap directory as the standard
+		 * icon_theme_dir, since the standard directory is themed */
+		g_object_get (G_OBJECT (dialog->priv->icon_entry), "pixmap_subdir",
+			      &(dialog->priv->icon_theme_dir), NULL);
+	}
+
+	/* do this after we set icon_theme_dir since this will launch a
+	 * callback */
+	gnome_icon_entry_set_filename (GNOME_ICON_ENTRY (dialog->priv->icon_entry),
+				       tmpstr);
+
+	g_free (tmpstr);
+	g_free (buffer);
+}
+
 /* Conform display to ditem */
 void
 panel_ditem_editor_sync_display (PanelDItemEditor *dialog)
@@ -1103,7 +1247,6 @@ panel_ditem_editor_sync_display (PanelDItemEditor *dialog)
 	gboolean              run_in_terminal;
 	GKeyFile             *key_file;
 	char                 *buffer;
-	char                 *tmpstr;
 	GtkTreeIter           iter;
 	GtkTreeModel         *model;
 	PanelDItemEditorType  buf_type;
@@ -1113,13 +1256,13 @@ panel_ditem_editor_sync_display (PanelDItemEditor *dialog)
 	key_file = dialog->priv->key_file;
 
 	/* Name */
-	buffer = panel_util_key_file_get_locale_string (key_file, "Name");
+	buffer = panel_key_file_get_locale_string (key_file, "Name");
 	gtk_entry_set_text (GTK_ENTRY (dialog->priv->name_entry),
 			    buffer ? buffer : "");
 	g_free (buffer);
 
 	/* Type */
-	type = panel_util_key_file_get_string (key_file, "Type");
+	type = panel_key_file_get_string (key_file, "Type");
 	if (!dialog->priv->combo_setuped) {
 		setup_combo (dialog->priv->type_combo,
 			     type_items, G_N_ELEMENTS (type_items),
@@ -1127,8 +1270,8 @@ panel_ditem_editor_sync_display (PanelDItemEditor *dialog)
 		dialog->priv->combo_setuped = TRUE;
 	}
 
-	run_in_terminal = panel_util_key_file_get_boolean (key_file, "Terminal",
-							   FALSE);
+	run_in_terminal = panel_key_file_get_boolean (key_file, "Terminal",
+						      FALSE);
 	editor_type = map_type_from_desktop_item (type, run_in_terminal);
 	g_free (type);
 
@@ -1149,10 +1292,10 @@ panel_ditem_editor_sync_display (PanelDItemEditor *dialog)
 
 	/* Command */
 	if (editor_type == PANEL_DITEM_EDITOR_TYPE_LINK)
-		buffer = panel_util_key_file_get_string (key_file, "URL");
+		buffer = panel_key_file_get_string (key_file, "URL");
 	else if (editor_type == PANEL_DITEM_EDITOR_TYPE_APPLICATION ||
 		 editor_type == PANEL_DITEM_EDITOR_TYPE_TERMINAL_APPLICATION)
-		buffer = panel_util_key_file_get_string (key_file, "Exec");
+		buffer = panel_key_file_get_string (key_file, "Exec");
 	else
 		buffer = NULL;
 
@@ -1161,32 +1304,15 @@ panel_ditem_editor_sync_display (PanelDItemEditor *dialog)
 	g_free (buffer);
 
 	/* Comment */
-	buffer = panel_util_key_file_get_locale_string (key_file, "Comment");
+	buffer = panel_key_file_get_locale_string (key_file, "Comment");
 	gtk_entry_set_text (GTK_ENTRY (dialog->priv->comment_entry),
 			    buffer ? buffer : "");
 	g_free (buffer);
 
 
 	/* Icon */
-	buffer = panel_util_key_file_get_locale_string (key_file, "Icon");
-	tmpstr = panel_find_icon (gtk_icon_theme_get_for_screen (gtk_widget_get_screen (GTK_WIDGET (dialog))),
-				  buffer, 48);
-	gnome_icon_entry_set_filename (GNOME_ICON_ENTRY (dialog->priv->icon_entry),
-				       tmpstr);
-
-	g_free (dialog->priv->icon_theme_dir);
-	if (buffer != NULL && !g_path_is_absolute (buffer)) {
-		/* this is a themed icon, see bug #119208 */
-		dialog->priv->icon_theme_dir = g_path_get_dirname (tmpstr);
-		/* FIXME: what about theme changes when the dialog is up */
-	} else {
-		/* use the default pixmap directory as the standard
-		 * icon_theme_dir, since the standard directory is themed */
-		g_object_get (G_OBJECT (dialog->priv->icon_entry), "pixmap_subdir",
-			      &(dialog->priv->icon_theme_dir), NULL);
-	}
-
-	g_free (tmpstr);
+	buffer = panel_key_file_get_locale_string (key_file, "Icon");
+	setup_icon_entry (dialog, buffer);
 	g_free (buffer);
 
 	if (dialog->priv->save_timeout != 0) {
@@ -1199,12 +1325,9 @@ static gboolean
 panel_ditem_editor_save (PanelDItemEditor *dialog,
 			 gboolean          report_errors)
 {
-	PanelDItemEditorType  type;
-	GKeyFile             *key_file;
-	const char           *const_buf;
-	char                 *buffer;
-	char                 *C_value;
-	GError               *error;
+	GKeyFile   *key_file;
+	const char *const_buf;
+	GError     *error;
 
 	g_return_val_if_fail (dialog != NULL, FALSE);
 	g_return_val_if_fail (dialog->priv->save_uri != NULL ||
@@ -1220,19 +1343,28 @@ panel_ditem_editor_save (PanelDItemEditor *dialog,
 	/* Verify that the required informations are set */
 	const_buf = gtk_entry_get_text (GTK_ENTRY (dialog->priv->name_entry));
 	if (const_buf == NULL || const_buf [0] == '\0') {
-		if (report_errors)
-			g_signal_emit (G_OBJECT (dialog),
-				       ditem_edit_signals[ERROR_REPORTED], 0,
-				       _("Could not save launcher"),
-				       _("The name of the launcher is not set."));
+		if (report_errors) {
+			if (!dialog->priv->type_directory)
+				g_signal_emit (G_OBJECT (dialog),
+					       ditem_edit_signals[ERROR_REPORTED], 0,
+					       _("Could not save launcher"),
+					       _("The name of the launcher is not set."));
+			else
+				g_signal_emit (G_OBJECT (dialog),
+					       ditem_edit_signals[ERROR_REPORTED], 0,
+					       _("Could not save directory properties"),
+					       _("The name of the directory is not set."));
+		}
 		return FALSE;
 	}
 
-	type = panel_ditem_editor_get_item_type (dialog);
 	const_buf = gtk_entry_get_text (GTK_ENTRY (dialog->priv->command_entry));
-	if (type != PANEL_DITEM_EDITOR_TYPE_DIRECTORY &&
+	if (!dialog->priv->type_directory &&
 	    (const_buf == NULL || const_buf [0] == '\0')) {
-		char *error;
+		PanelDItemEditorType  type;
+		char                 *error;
+
+		type = panel_ditem_editor_get_item_type (dialog);
 
 		switch (type) {
 		case PANEL_DITEM_EDITOR_TYPE_APPLICATION:
@@ -1256,45 +1388,9 @@ panel_ditem_editor_save (PanelDItemEditor *dialog,
 
 	key_file = dialog->priv->key_file;
 
-	/* Make sure we set the "C" locale strings to the terms we set here.
-	 * This is so that if the user logs into another locale they get their
-	 * own description there rather then empty. It is not the C locale
-	 * however, but the user created this entry herself so it's OK */
-	C_value = panel_util_key_file_get_string (key_file, "Name");
-	if (C_value == NULL || C_value [0] == '\0') {
-		buffer = panel_util_key_file_get_locale_string (key_file,
-								"Name");
-		if (buffer) {
-			panel_util_key_file_set_string (key_file, "Name",
-							buffer);
-			g_free (buffer);
-		}
-	}
-	g_free (C_value);
-
-	C_value = panel_util_key_file_get_string (key_file, "Comment");
-	if (C_value == NULL || C_value [0] == '\0') {
-		buffer = panel_util_key_file_get_locale_string (key_file,
-								"Comment");
-		if (buffer) {
-			panel_util_key_file_set_string (key_file, "Comment",
-							buffer);
-			g_free (buffer);
-		}
-	}
-	g_free (C_value);
-
-	C_value = panel_util_key_file_get_string (key_file, "Icon");
-	if (C_value == NULL || C_value [0] == '\0') {
-		buffer = panel_util_key_file_get_locale_string (key_file,
-								"Icon");
-		if (buffer) {
-			panel_util_key_file_set_string (key_file, "Icon",
-							buffer);
-			g_free (buffer);
-		}
-	}
-	g_free (C_value);
+	panel_key_file_ensure_C_key (key_file, "Name");
+	panel_key_file_ensure_C_key (key_file, "Comment");
+	panel_key_file_ensure_C_key (key_file, "Icon");
 
 	if (dialog->priv->save_uri) {
 		char *uri;
@@ -1310,9 +1406,9 @@ panel_ditem_editor_save (PanelDItemEditor *dialog,
 
 	/* And now, try to save */
 	error = NULL;
-	panel_util_key_file_to_file (dialog->priv->key_file,
-				     dialog->priv->uri,
-				     &error);
+	panel_key_file_to_file (dialog->priv->key_file,
+				dialog->priv->uri,
+				&error);
 	if (error != NULL) {
 		if (report_errors)
 			g_signal_emit (G_OBJECT (dialog),
@@ -1406,41 +1502,41 @@ panel_ditem_editor_revert (PanelDItemEditor *dialog)
 	for (i = 0; i < G_N_ELEMENTS (revert_keys); i++) {
 		if (revert_keys [i].type == G_TYPE_STRING) {
 			if (revert_keys [i].locale) {
-				string = panel_util_key_file_get_locale_string (
+				string = panel_key_file_get_locale_string (
 						revert_key_file,
 						revert_keys [i].key);
 				if (string == NULL)
-					panel_util_key_file_remove_locale_key (
+					panel_key_file_remove_all_locale_key (
 							key_file,
 							revert_keys [i].key);
 				else
-					panel_util_key_file_set_locale_string (
+					panel_key_file_set_locale_string (
 							key_file,
 							revert_keys [i].key,
 							string);
 			} else {
-				string = panel_util_key_file_get_string (
+				string = panel_key_file_get_string (
 						revert_key_file,
 						revert_keys [i].key);
 				if (string == NULL)
-					panel_util_key_file_remove_key (
+					panel_key_file_remove_key (
 							key_file,
 							revert_keys [i].key);
 				else
-					panel_util_key_file_set_string (
+					panel_key_file_set_string (
 							key_file,
 							revert_keys [i].key,
 							string);
 			}
 			g_free (string);
 		} else if (revert_keys [i].type == G_TYPE_BOOLEAN) {
-			boolean = panel_util_key_file_get_boolean (
+			boolean = panel_key_file_get_boolean (
 					revert_key_file,
 					revert_keys [i].key,
 					revert_keys [i].default_value);
-			panel_util_key_file_set_boolean (key_file,
-							 revert_keys [i].key,
-							 boolean);
+			panel_key_file_set_boolean (key_file,
+						    revert_keys [i].key,
+						    boolean);
 		} else {
 			g_assert_not_reached ();
 		}
@@ -1452,9 +1548,10 @@ panel_ditem_editor_revert (PanelDItemEditor *dialog)
 		if (dialog->priv->save_timeout != 0)
 			g_source_remove (dialog->priv->save_timeout);
 
-		dialog->priv->save_timeout = g_timeout_add (SAVE_FREQUENCY,
-							    panel_ditem_editor_save_timeout,
-							    dialog);
+		dialog->priv->save_timeout = g_timeout_add_seconds (
+							SAVE_FREQUENCY,
+							panel_ditem_editor_save_timeout,
+							dialog);
 	}
 }
 
@@ -1478,33 +1575,33 @@ panel_ditem_editor_set_revert (PanelDItemEditor *dialog)
 	for (i = 0; i < G_N_ELEMENTS (revert_keys); i++) {
 		if (revert_keys [i].type == G_TYPE_STRING) {
 			if (revert_keys [i].locale) {
-				string = panel_util_key_file_get_locale_string (
+				string = panel_key_file_get_locale_string (
 						key_file,
 						revert_keys [i].key);
 				if (string != NULL)
-					panel_util_key_file_set_locale_string (
+					panel_key_file_set_locale_string (
 							revert_key_file,
 							revert_keys [i].key,
 							string);
 			} else {
-				string = panel_util_key_file_get_string (
+				string = panel_key_file_get_string (
 						key_file,
 						revert_keys [i].key);
 				if (string != NULL)
-					panel_util_key_file_set_string (
+					panel_key_file_set_string (
 							revert_key_file,
 							revert_keys [i].key,
 							string);
 			}
 			g_free (string);
 		} else if (revert_keys [i].type == G_TYPE_BOOLEAN) {
-			boolean = panel_util_key_file_get_boolean (
+			boolean = panel_key_file_get_boolean (
 					key_file,
 					revert_keys [i].key,
 					revert_keys [i].default_value);
-			panel_util_key_file_set_boolean (revert_key_file,
-							 revert_keys [i].key,
-							 boolean);
+			panel_key_file_set_boolean (revert_key_file,
+						    revert_keys [i].key,
+						    boolean);
 		} else {
 			g_assert_not_reached ();
 		}
@@ -1537,10 +1634,10 @@ panel_ditem_editor_load_uri (PanelDItemEditor  *dialog,
 
 	key_file = g_key_file_new ();
 
-	if (!panel_util_key_file_load_from_uri (key_file,
-						dialog->priv->uri,
-						G_KEY_FILE_KEEP_COMMENTS|G_KEY_FILE_KEEP_TRANSLATIONS,
-						error)) {
+	if (!panel_key_file_load_from_uri (key_file,
+					   dialog->priv->uri,
+					   G_KEY_FILE_KEEP_COMMENTS|G_KEY_FILE_KEEP_TRANSLATIONS,
+					   error)) {
 		g_key_file_free (key_file);
 		return FALSE;
 	}
@@ -1554,11 +1651,12 @@ panel_ditem_editor_load_uri (PanelDItemEditor  *dialog,
 	return TRUE;
 }
 
-GtkWidget *
-panel_ditem_editor_new (GtkWindow   *parent,
-			GKeyFile    *key_file,
-			const char  *uri,
-			const char  *title)
+static GtkWidget *
+panel_ditem_editor_new_full (GtkWindow   *parent,
+			     GKeyFile    *key_file,
+			     const char  *uri,
+			     const char  *title,
+			     gboolean     type_directory)
 {
 	GtkWidget *dialog;
 
@@ -1566,6 +1664,7 @@ panel_ditem_editor_new (GtkWindow   *parent,
 			       "title", title,
 			       "keyfile", key_file,
 			       "uri", uri,
+			       "type-directory", type_directory,
 			       NULL);
 
 	if (parent)
@@ -1574,6 +1673,26 @@ panel_ditem_editor_new (GtkWindow   *parent,
 	return dialog;
 }
 
+GtkWidget *
+panel_ditem_editor_new (GtkWindow   *parent,
+			GKeyFile    *key_file,
+			const char  *uri,
+			const char  *title)
+{
+	return panel_ditem_editor_new_full (parent, key_file, uri,
+					    title, FALSE);
+}
+
+GtkWidget *
+panel_ditem_editor_new_directory (GtkWindow   *parent,
+				  GKeyFile    *key_file,
+				  const char  *uri,
+				  const char  *title)
+{
+	return panel_ditem_editor_new_full (parent, key_file, uri,
+					    title, TRUE);
+}
+
 static void
 panel_ditem_editor_set_key_file (PanelDItemEditor *dialog,
 				 GKeyFile         *key_file)
@@ -1613,6 +1732,20 @@ panel_ditem_editor_set_uri (PanelDItemEditor *dialog,
 	g_object_notify (G_OBJECT (dialog), "uri");
 }
 
+static void
+panel_ditem_editor_set_type_directory (PanelDItemEditor *dialog,
+				       gboolean          type_directory)
+{
+	g_return_if_fail (PANEL_IS_DITEM_EDITOR (dialog));
+
+	if (dialog->priv->type_directory == type_directory)
+		return;
+
+	dialog->priv->type_directory = type_directory;
+
+	g_object_notify (G_OBJECT (dialog), "type-directory");
+}
+
 GKeyFile *
 panel_ditem_editor_get_key_file (PanelDItemEditor *dialog)
 {
@@ -1637,6 +1770,14 @@ panel_ditem_editor_get_uri (PanelDItemEditor *dialog)
 	return dialog->priv->uri;
 }
 
+static gboolean
+panel_ditem_editor_get_type_directory (PanelDItemEditor *dialog)
+{
+	g_return_val_if_fail (PANEL_IS_DITEM_EDITOR (dialog), FALSE);
+
+	return dialog->priv->type_directory;
+}
+
 void
 panel_ditem_register_save_uri_func (PanelDItemEditor  *dialog,
 				    PanelDitemSaveUri  save_uri,
