@@ -1,6 +1,7 @@
 #ifndef PANEL_UTIL_H
 #define PANEL_UTIL_H
 
+#include <gio/gio.h>
 #include <gtk/gtk.h>
 
 G_BEGIN_DECLS
@@ -93,37 +94,6 @@ GdkPixbuf *panel_util_cairo_rgbdata_to_pixbuf (unsigned char *data,
 					       int            width,
 					       int            height);
 
-GKeyFile *panel_util_key_file_new_desktop  (void);
-gboolean  panel_util_key_file_to_file      (GKeyFile       *keyfile,
-					    const gchar    *file,
-					    GError        **error);
-gboolean panel_util_key_file_load_from_uri (GKeyFile       *keyfile,
-					    const gchar    *uri,
-					    GKeyFileFlags   flags,
-					    GError        **error);
-gboolean panel_util_key_file_get_boolean   (GKeyFile       *keyfile,
-					    const gchar    *key,
-					    gboolean        default_value);
-#define panel_util_key_file_get_string(key_file, key) \
-	 g_key_file_get_string (key_file, "Desktop Entry", key, NULL)
-#define panel_util_key_file_get_locale_string(key_file, key) \
-	 g_key_file_get_locale_string(key_file, "Desktop Entry", key, NULL, NULL)
-#define panel_util_key_file_set_boolean(key_file, key, value) \
-	 g_key_file_set_boolean (key_file, "Desktop Entry", key, value)
-#define panel_util_key_file_set_string(key_file, key, value) \
-	 g_key_file_set_string (key_file, "Desktop Entry", key, value)
-void    panel_util_key_file_set_locale_string (GKeyFile    *keyfile,
-					       const gchar *key,
-					       const gchar *value);
-#define panel_util_key_file_remove_key(key_file, key) \
-	g_key_file_remove_key (key_file, "Desktop Entry", key, NULL)
-void panel_util_key_file_remove_locale_key (GKeyFile    *keyfile,
-					    const gchar *key);
-void panel_util_key_file_remove_all_locale_key (GKeyFile    *keyfile,
-						const gchar *key);
-void panel_util_key_file_ensure_C_key (GKeyFile   *keyfile,
-				       const char *key);
-
 char *guess_icon_from_exec (GtkIconTheme *icon_theme,
 			    GKeyFile     *key_file);
 
@@ -131,6 +101,14 @@ const char *panel_util_get_vfs_method_display_name (const char *method);
 char *panel_util_get_label_for_uri (const char *text_uri);
 char *panel_util_get_icon_for_uri (const char *text_uri);
 
+void panel_util_set_tooltip_text (GtkWidget  *widget,
+				  const char *text);
+
+char *panel_util_get_icon_name_from_g_icon (GIcon *gicon);
+GdkPixbuf *panel_util_get_pixbuf_from_g_loadable_icon (GIcon *gicon,
+						       int    size);
+GFile *panel_util_get_file_optional_homedir (const char *location);
+
 G_END_DECLS
 
 #endif /* PANEL_UTIL_H */
